//****************************************************************************
// File: mainwnd.c
//
// Purpose: Main overlapped window's message handler
//
// Functions:
//    MainWndProc() - message handler for main overlapped window
//    DoCommands() - called by MainWndProc to handle all WM_COMMAND messages
//    CreateFixedWidthFont() - Creates a new font
//    Draw3DRect() - Draws a 3-D rectangle
//    DrawLabels() - Draws labels for UI
//
// Development Team:
//     Greg Binkerd
//
// Written by Microsoft Developer Support
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include <string.h>
#include <commdlg.h>
#include "global.h"
#include "about.h"
#include "mainwnd.h"


// Local function prototypes.
LRESULT DoCommands (HWND hWnd,
                    UINT message,
                    WPARAM wParam,
                    LPARAM lParam);

//****************************************************************************
// Function: MainWndProc
//
// Purpose: Message handler for main overlapped window.
//
// Parameters:
//    hWnd    == Handle to this window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author        Reason
//           6/08/94                  Created
//****************************************************************************

LRESULT CALLBACK MainWndProc (HWND hWnd,
                              UINT message,
                              WPARAM wParam,
                              LPARAM lParam)
{
   switch (message)
   {
   static HFONT hListFont;
   
   case WM_CREATE:  
      {
      TEXTMETRIC tm;
      HDC hdc;
      
      // Create the listbox.
      ghWndList  = CreateWindow("listbox", 
                       NULL,        // Text for window title bar.
                       WS_BORDER|WS_CHILD|WS_VISIBLE|LBS_NOINTEGRALHEIGHT|WS_VSCROLL,
                       0,0,0,0,
                       hWnd,
                       NULL,           
                       ghInst,      // This instance owns this window.   
                       NULL);          // Pointer not used.               
      hdc = GetDC(NULL);
      GetTextMetrics(hdc,&tm);
      ReleaseDC(NULL,hdc);
      hListFont = CreateFixedWidthFont(tm.tmHeight/5*6,FW_NORMAL); //little larger than normal size
      SendMessage(ghWndList,WM_SETFONT,hListFont,0L);
      break;
      }

      // Dispatch WM_COMMAND messages to our command handler, DoCommands().
      case WM_COMMAND:
         return DoCommands(hWnd, message, wParam, lParam);

      case WM_PAINT:
         {
         HDC hdc;
         PAINTSTRUCT ps;
         RECT rc;
         HFONT hFont, hOldFont;
         
         GetClientRect(hWnd,&rc);
         if (rc.right < WINDOW_THRESHOLD)
             rc.right = WINDOW_THRESHOLD;
         if (rc.bottom < WINDOW_THRESHOLD)
             rc.bottom = WINDOW_THRESHOLD;
         // Move the listbox window to the bottom of the main window
         MoveWindow(ghWndList,LIST_BORDER,rc.bottom/4*3,rc.right-LIST_BORDER*2,
                    rc.bottom/4-LIST_BORDER,TRUE);
         hdc = BeginPaint(hWnd, &ps);
         SetMapMode(hdc,MM_ANISOTROPIC);
         SetWindowExt(hdc,1000,1000);
         SetViewportExt(hdc,rc.right,rc.bottom);
         // Draw the rectangles to contain font information
         Draw3DRect(hdc,10,50,500,400);
         Draw3DRect(hdc,510,50,990,700);
         Draw3DRect(hdc,10,450,500,700);

         hFont = CreateFixedWidthFont(40,FW_NORMAL);
         hOldFont = SelectObject(hdc,hFont);
         SetBkMode(hdc,TRANSPARENT);
         rc.left = 0;
         rc.top = 700;
         rc.right = 1000;
         rc.bottom = 750;
         DrawText(hdc,"Line(s) Added to WIN.INI",24,&rc,
                       DT_CENTER|DT_SINGLELINE|DT_VCENTER);
         rc.left = 10;
         rc.top = 0;
         rc.right = 500;
         rc.bottom = 50;    
         DrawText(hdc,"Font Information",16,&rc,
                       DT_CENTER|DT_SINGLELINE|DT_VCENTER);
         rc.left = 510;
         rc.top = 0;
         rc.right = 990;
         rc.bottom = 50;
         DrawText(hdc,"Text Metrics",12,&rc,
                       DT_CENTER|DT_SINGLELINE|DT_VCENTER);
         rc.left = 10;
         rc.top = 400;
         rc.right = 500;
         rc.bottom = 450;
         DrawText(hdc,"Sample Text",11,&rc,
                       DT_CENTER|DT_SINGLELINE|DT_VCENTER);
         // Draw labels for TEXTMETRIC and other font information
         DrawLabels(hdc);
         DeleteObject(SelectObject(hdc,hOldFont));
         if (ghCurrFont)
            DrawFontInfo(hdc);
         EndPaint(hWnd, &ps);
         break;
         }
 
      // On WM_DESTROY, terminate this app by posting a WM_QUIT message.
      case WM_DESTROY:
         if (ghCurrFont)
            DeleteObject(ghCurrFont);
         DeleteObject(hListFont);
         PostQuitMessage(0);
         break;


      // We didn't handle, pass to DefWindowProc.

      default:
         return DefWindowProc(hWnd, message, wParam, lParam);
   }

   return (NULL);
}


//****************************************************************************
// Function: DoCommands
//
// Purpose: Called by MainWndProc() to handle all WM_COMMAND messages.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns : Depends on message.
//
// Comments:
//
// History:  Date       Author        Reason
//           6/08/94                  Created
//****************************************************************************

LRESULT DoCommands (HWND hWnd,
                    UINT message,
                    WPARAM wParam,
                    LPARAM lParam)
{
   static char szFile[256];
   static char szFileTitle[256];

   switch (wParam)
   {
      case IDM_INSTALLRASTERVECTOR:
      case IDM_INSTALLTRUETYPE:
         {  
         BOOL bTrueType = (wParam == IDM_INSTALLTRUETYPE);
         OPENFILENAME ofn;
         char  szFilter[256];

         szFileTitle[0] = '\0';
         szFile[0] = '\0';
         if (bTrueType)
            wsprintf(szFilter,"TrueType(*.TTF)%c*.TTF%c%c",'\0','\0','\0');
         else
            wsprintf(szFilter,"Raster/Vector(*.FON)%c*.FON%c%c",'\0','\0','\0');
         /* Set all structure members to zero. */
         memset(&ofn, 0, sizeof(OPENFILENAME));
         ofn.lStructSize = sizeof(OPENFILENAME);
         ofn.hwndOwner = hWnd;
         ofn.lpstrFilter = szFilter;
         ofn.nFilterIndex = 0;
         ofn.lpstrFile= szFile;
         ofn.nMaxFile = sizeof(szFile);
         ofn.lpstrFileTitle = szFileTitle;
         ofn.nMaxFileTitle = sizeof(szFileTitle);
         ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
 
         // Get the file name for the font to install
         if (GetOpenFileName(&ofn)) 
            {
            if (bTrueType)
               InstallTrueType((PSTR)szFile,(PSTR)szFileTitle);
            else
               InstallRasterVector((LPSTR)szFile,(LPSTR)szFileTitle);
            InvalidateRect(hWnd,NULL,TRUE);
            }
         break;
         }

      // Put up the About box.
      case IDM_ABOUT:
      {
         FARPROC lpDlgProc;

         lpDlgProc = MakeProcInstance(AboutDlg, ghInst);

         DialogBox(ghInst,             // current instance
                   AboutBoxName,       // resource to use 
                   hWnd,               // parent handle   
                   lpDlgProc);         // About() instance address

         FreeProcInstance(lpDlgProc);
         break;
      }


      // User picked File.Exit so post WM_CLOSE, causing the default
      // window procedure to call DestroyWindow(). We could process
      // WM_CLOSE for confirmation from user that we should exit.

      case IDM_EXIT:
         PostMessage(hWnd, WM_CLOSE, 0, 0L);
         break;


      // Must be some system command -- pass it on to the default
      //  window procedure.

      default:
         return DefWindowProc(hWnd, message, wParam, lParam);
   }

   return (NULL);
}
 

//****************************************************************************
// Function: CreateFixedWidthFont
//
// Purpose: Creates a fixed width (Courier New) font.
//
// Parameters:
//    nHeight   == Height of font.
//    nBoldness == Boldness of font.
//
// Returns: Handle of newly created logical font
//
// Comments:
//
// History:  Date       Author        Reason
//           3/7/95     GGB           Created
//****************************************************************************

HFONT CreateFixedWidthFont(int nHeight, int nBoldness)
{
   LOGFONT lf;

   memset(&lf,0,sizeof(LOGFONT));
   lf.lfHeight = -nHeight;
   lf.lfWeight = nBoldness;
   lf.lfCharSet = ANSI_CHARSET;
   lf.lfOutPrecision = OUT_TT_ONLY_PRECIS;
   lf.lfPitchAndFamily = FIXED_PITCH|FF_MODERN;
   lstrcpy(lf.lfFaceName,"Courier New");
   return (CreateFontIndirect(&lf));
}

 
//****************************************************************************
// Function: Draw3DRect
//
// Purpose: Draws a 3D rectangle for the User Interface
//
// Parameters:
//    hdc       == Device context to draw rectangle on
//    nLeft, nTop, nRight, nBottom   == Dimensions of rectangle
//
// Returns: none
//
// Comments:
//
// History:  Date       Author        Reason
//           3/7/95     GGB           Created
//****************************************************************************

void Draw3DRect(HDC hdc, int nLeft, int nTop, int nRight, int nBottom)
{  
   HPEN hPen, hOldPen;
   POINT ptRc[2];
   
   ptRc[0].x = nLeft;
   ptRc[0].y = nTop;
   ptRc[1].x = nRight;
   ptRc[1].y = nBottom;
   
   LPtoDP(hdc,ptRc,2);
   SaveDC(hdc);
   SetMapMode(hdc,MM_TEXT);
   SelectObject(hdc,GetStockObject(WHITE_BRUSH));
   Rectangle(hdc,ptRc[0].x+1,ptRc[0].y+1,ptRc[1].x,ptRc[1].y);
   SelectObject(hdc,GetStockObject(WHITE_PEN));
   MoveTo(hdc,ptRc[1].x,ptRc[0].y);
   LineTo(hdc,ptRc[1].x,ptRc[1].y);
   LineTo(hdc,ptRc[0].x,ptRc[1].y);

   hPen = CreatePen(PS_SOLID,0,RGB(128,128,128));
   hOldPen = SelectObject(hdc,hPen);
   MoveTo(hdc,ptRc[0].x,ptRc[1].y);
   LineTo(hdc,ptRc[0].x,ptRc[0].y);
   LineTo(hdc,ptRc[1].x,ptRc[0].y);
   DeleteObject(SelectObject(hdc,hOldPen));
   RestoreDC(hdc,-1);
}

 
//****************************************************************************
// Function: DrawLabels
//
// Purpose: Draws labels for the font information being displayed
//
// Parameters:
//    HDC   == Device context to draw labels on
//
// Returns: none
//
// Comments:
//
// History:  Date       Author        Reason
//           3/7/95     GGB           Created
//****************************************************************************

void DrawLabels(HDC hdc)
{
  HFONT hFont, hOldFont;
  
  hFont = CreateFixedWidthFont(32,FW_NORMAL);
  hOldFont = SelectObject(hdc,hFont);
  TextOut(hdc,20,80,"Font Name:",10);
  TextOut(hdc,20,130,"Font Type:",10);
  TextOut(hdc,20,180,"File:",5);
  TextOut(hdc,20,230,"FOT File:",9);
  TextOut(hdc,20,280,"Scalable?",9);
  TextOut(hdc,20,330,"Pitch:",6);
  DeleteObject(SelectObject(hdc,hOldFont));
  hFont = CreateFixedWidthFont(29,FW_NORMAL);
  hOldFont = SelectObject(hdc,hFont);
  TextOut(hdc,520,53,"tmHeight:",9);
  TextOut(hdc,520,85,"tmAscent:",9);
  TextOut(hdc,520,117,"tmDescent:",10);
  TextOut(hdc,520,149,"tmInternalLeading:",18);
  TextOut(hdc,520,181,"tmExternalLeading:",18);
  TextOut(hdc,520,213,"tmAveCharWidth:",15);
  TextOut(hdc,520,245,"tmMaxCharWidth:",15);
  TextOut(hdc,520,277,"tmWeight:",9);
  TextOut(hdc,520,309,"tmItalic:",9);
  TextOut(hdc,520,341,"tmUnderlined:",13);
  TextOut(hdc,520,373,"tmStruckOut:",12);
  TextOut(hdc,520,405,"tmFirstChar:",12);
  TextOut(hdc,520,437,"tmLastChar:",11);
  TextOut(hdc,520,469,"tmDefaultChar:",14);
  TextOut(hdc,520,501,"tmBreakChar:",12);
  TextOut(hdc,520,533,"tmPitchAndFamily:",17);
  TextOut(hdc,520,565,"tmCharSet:",10);
  TextOut(hdc,520,597,"tmOverhang:",11);
  TextOut(hdc,520,629,"tmDigitizedAspectX:",19);
  TextOut(hdc,520,661,"tmDigitizedAspectY:",19);
  DeleteObject(SelectObject(hdc,hOldFont));
} 